package org.intranet.games.minesweeper.swing;

import javax.swing.JComponent;
import org.intranet.games.minesweeper.*;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.FlowLayout;

/** 
*
* @author Chris Dailey
*/
class PlayingFieldView
  extends JComponent
  implements PlayingField.PlayingFieldListener
{
  private PlayingField playingField;
  private NumberDisplay counter = new NumberDisplay(3);
  private NumberDisplay timer = new NumberDisplay(3);
  private javax.swing.JPanel field = new javax.swing.JPanel();
  private GridLayout fieldLayout = new GridLayout();
  private GameStatusButton restart = new GameStatusButton();

  public void timerValueChanged(int s)
  {
    timer.setValue(s);
  }

  PlayingFieldView(PlayingField pf)
  {
    super();
    playingField = pf;
    setLayout(new BorderLayout());

    javax.swing.JPanel topBar = new javax.swing.JPanel();
    topBar.setLayout(new BorderLayout());

    topBar.add(counter, BorderLayout.WEST);

    restart.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent ae) {
        playingField.init();
      }
    });
    javax.swing.JPanel restartPanel = new javax.swing.JPanel();
    restartPanel.add(restart);
    topBar.add(restartPanel, BorderLayout.CENTER);

    topBar.add(timer, BorderLayout.EAST);

    add(topBar, BorderLayout.NORTH);

    field.setLayout(fieldLayout);
    playingField.addPlayingFieldListener(this);
    playingFieldConfigurationChanged(null);
    add(field, BorderLayout.CENTER);
  }

  public void playingFieldConfigurationChanged(
    PlayingField.PlayingFieldEvent pfe)
  {
    field.removeAll();
    int w = playingField.getWidth();
    int h = playingField.getHeight();
    fieldLayout.setColumns(playingField.getWidth());
    fieldLayout.setRows(playingField.getHeight());
    // GridLayout goes across rows and then down to the next row, so we
    // need to build these with x in the inner loop instead of y.
    for (int y = 0; y < h; y++)
    {
      for (int x = 0; x < w; x++)
      {
        Square square = playingField.getSquare(x, y);
        SquareView squareView = new SquareView(square);
        field.add(squareView);
      }
    }

    // Now get ready to meet the neighbors
    for (int y = 0; y < h; y++)
    {
      for (int x = 0; x < w; x++)
      {
        // look at neighbors
        java.util.List neighbors = new java.util.LinkedList();
        for (int y1 = -1; y1 <= 1; y1++)
        {
          int neighborY = y1 + y;
          if (neighborY < 0 || neighborY >= h)
            continue;
          for (int x1 = -1; x1 <= 1; x1++)
          {
            int neighborX = x1 + x;
            if (neighborX < 0 || neighborX >= w)
              continue;
            if (x1 == 0 && y1 == 0)
              continue;
            SquareView neighbor = (SquareView)field.getComponent(neighborX + neighborY * w);
            neighbors.add(neighbor);
          }
        }
        SquareView squareView = (SquareView)field.getComponent(x + y * w);
        Object[] neighborObjects = neighbors.toArray();
        SquareView[] neighborArray = new SquareView[neighborObjects.length];
        for (int i = neighborObjects.length - 1; i >= 0; i--)
          neighborArray[i] = (SquareView)neighborObjects[i];
        squareView.setNeighbors(neighborArray);
      }
    }
  }

  public void playingFieldCounterUpdated(int value)
  {
    if (playingField.getStatus() != PlayingField.Status.initted)
      counter.setValue(value);
  }
  public void playingFieldStatusChanged(PlayingField.Status newStatus)
  {
    restart.setStatus(newStatus);
    if (newStatus == PlayingField.Status.initted)
    {
      counter.setValue(Integer.MAX_VALUE);
      timer.setValue(Integer.MAX_VALUE);
      field.repaint();
    }
    if (newStatus == PlayingField.Status.won)
    {
      java.awt.Component[] squares = field.getComponents();
      for (int i = squares.length - 1; i >= 0; i--)
        ((SquareView)squares[i]).gameOver();
      repaint();
    }
    if (newStatus == PlayingField.Status.lost)
    {
      java.awt.Component[] squares = field.getComponents();
      for (int i = squares.length - 1; i >= 0; i--)
        ((SquareView)squares[i]).gameOver();
      repaint();
    }
  }
}