package org.intranet.games.minesweeper.swing;

import javax.swing.JComponent;
import org.intranet.games.minesweeper.*;

/** 
*
* @author Chris Dailey
*/
class SquareView
  extends JComponent
  implements Square.SquareListener, java.awt.event.MouseListener
{
  public static final java.awt.Dimension prefSize =
    new java.awt.Dimension(20, 20);
  public static final java.awt.Dimension labelPrefSize =
    new java.awt.Dimension(15, 15);

  public static final javax.swing.border.Border raisedBorder =
    javax.swing.BorderFactory.createRaisedBevelBorder();
  public static final javax.swing.border.Border loweredBorder =
    javax.swing.BorderFactory.createLoweredBevelBorder();
  public static final javax.swing.border.Border lineBorder =
    javax.swing.BorderFactory.createLineBorder(java.awt.Color.gray, 1);

  private static final class PictureType
  {
    String name;
    public static final PictureType covered = new PictureType("covered");
    public static final PictureType flagged = new PictureType("flagged");
    public static final PictureType question = new PictureType("question");
    public static final PictureType uncovered = new PictureType("uncovered");
    public static final PictureType mine = new PictureType("mine");
    public static final PictureType badFlag = new PictureType("badflag");
    // public static final PictureType none = new PictureType("none");
    PictureType(String s) { name = s; }
    public String toString() { return name; }
  }

  private void setPictureNumber(int i)
  {
    if (i > 0 && i <= 8)
      label.setText(String.valueOf(i));
    else
      label.setText("");
  }
  private void setPictureType(PictureType p)
  {
    if (p == PictureType.covered || // p == PictureType.none ||
      p == PictureType.uncovered)
    {
      label.setText("");
    }
    else if (p == PictureType.flagged)
      label.setText("F");
    else if (p == PictureType.question)
      label.setText("?");
    else if (p == PictureType.mine)
      label.setText("O");
    else if (p == PictureType.badFlag)
      label.setText("X");
  }

  private Square square;
  private SquareView[] neighbors;
  private javax.swing.JLabel label = new javax.swing.JLabel();
  SquareView(Square s)
  {
    super();
    setLayout(new java.awt.BorderLayout());
    this.addMouseListener(this);
    setBorder(raisedBorder);
    label.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
    add(label, java.awt.BorderLayout.CENTER);
    square = s;
    square.addSquareListener(this);
    setPreferredSize(prefSize);
  }

  void setNeighbors(SquareView[] n)
  {
    neighbors = n;
  }

  public void gameOver()
  {
    Square.State state = square.getState();
    if ((state == Square.State.covered || state == Square.State.question) &&
      square.hasMine())
    {
      setPictureType(PictureType.mine);
    }
    else if (state == Square.State.flagged && !square.hasMine())
      setPictureType(PictureType.badFlag);
  }

  public void squarePropertyChanged(Square.SquareEvent se)
  {
    Square.State state = square.getState();
    if (getBorder() == raisedBorder && state == Square.State.uncovered)
    {
      // uncovering
      String value = null;
      if (square.hasMine())
        setPictureType(PictureType.mine);
      else
        setPictureNumber(square.getNumber());
      setBorder(lineBorder); // causes a validate
    }
    else if (getBorder() == raisedBorder)
    {
      // flagging/marking
      if (state == Square.State.flagged)
        setPictureType(PictureType.flagged);
      else if (state == Square.State.question)
        setPictureType(PictureType.question);
      else
        setPictureType(PictureType.covered);
    }
    else if (getBorder() == lineBorder && state == Square.State.covered)
    {
      // initting
      setBorder(raisedBorder);
      setPictureType(PictureType.covered);
    }
  }

  void threaten()
  {
    if (square.getState() == Square.State.covered)
    {
      setBorder(loweredBorder);
    }
  }

  void unthreaten()
  {
    if (square.getState() == Square.State.covered)
    {
      setBorder(raisedBorder);
    }
  }

  Square getSquare() { return square; }

  public void mouseClicked(java.awt.event.MouseEvent me)
  {
    if (square.getPlayingField().isOver())
      return;
    if ((me.getModifiers() & me.BUTTON3_MASK) != 0)
    {
      if (square.getState() == Square.State.uncovered)
        square.uncoverNeighbors();
      else
        square.mark();
    }
    else if ((me.getModifiers() & me.BUTTON1_MASK) != 0)
      square.uncover();
    if (false)
      org.intranet.games.minesweeper.commandline.MineSweeper.printPlayingField(
        square.getPlayingField());
  }
  public void mousePressed(java.awt.event.MouseEvent me)
  {
    if (square.getPlayingField().isOver())
      return;
    Square.State state = square.getState();
    if (state == Square.State.covered)
      setBorder(loweredBorder);
    if ((me.getModifiers() & me.BUTTON3_MASK) != 0 &&
      state == Square.State.uncovered)
    {
      for(int i = neighbors.length - 1; i >= 0; i--)
      {
        SquareView neighbor = neighbors[i];
        if (neighbor.getSquare().getState() == Square.State.covered)
          neighbor.threaten();
      }
      return;
    }
  }
  public void mouseReleased(java.awt.event.MouseEvent me)
  {
    if (square.getPlayingField().isOver())
      return;
    Square.State state = square.getState();
    if ((me.getModifiers() & me.BUTTON3_MASK) != 0 &&
      state == Square.State.uncovered)
    {
      for(int i = neighbors.length - 1; i >= 0; i--)
      {
        SquareView neighbor = neighbors[i];
        if (neighbor.getSquare().getState() == Square.State.covered)
          neighbor.unthreaten();
      }
      return;
    }
    if (state == Square.State.covered)
      setBorder(raisedBorder);
  }
  public void mouseEntered(java.awt.event.MouseEvent me) { }
  public void mouseExited(java.awt.event.MouseEvent me)
  {
    if (square.getState() != Square.State.uncovered &&
      getBorder() == loweredBorder)
    {
      setBorder(raisedBorder);
    }
  }
}